@@ -1,3 +1,55 @@
+//! Quick Csv reader which performs **very** well.
+//! 
+//! ## Example
+//! 
+//! First, create a `Csv` from a `BufRead` reader, a file or a string
+//! 
+//! ```rust
+//! extern crate quick_csv;
+//! 
+//! fn main() {
+//!     let data = "a,b\r\nc,d\r\ne,f";
+//!     let csv = quick_csv::Csv::from_string(data);
+//!     for row in csv.into_iter() {
+//!         // work on csv row ...
+//!         if let Ok(_) = row {
+//!             println!("new row!");
+//!         } else {
+//!             println!("cannot read next line");
+//!         }
+//!     }
+//! }
+//! ```
+//! 
+//! `Row` is on the other hand provides 3 methods to access csv columns:
+//! - `columns`: 
+//!   - iterator over columns.
+//!   - Iterator item is a `&str`, which means you only have to `parse()` it to the needed type and you're done
+//! 
+//!   ```rust
+//!   # let row = quick_csv::Csv::from_string("a,b,c,d,e,38,f").next().unwrap().unwrap();
+//!   let mut cols = row.columns().expect("cannot convert to utf8");
+//!   let fifth = cols.nth(5).unwrap().parse::<f64>().unwrap();
+//!   println!("Doubled fifth column: {}", fifth * 2.0);
+//!   ```
+//! 
+//! - `decode`:
+//!   - deserialize into you `Decodable` struct, a-la rust-csv.
+//!   - most convenient way to deal with your csv data
+//! 
+//!   ```rust
+//!   let row = quick_csv::Csv::from_string("a,b,54").next().unwrap().unwrap();
+//!   if let Ok((col1, col2, col3)) = row.decode::<(String, u64, f64)>() {
+//!       println!("col1: '{}', col2: {}, col3: {}", col1, col2, col3);
+//!   }
+//!   ``` 
+//! 
+//! - `bytes_columns`:
+//!   - similar to `columns` but columns are of type `&[u8]`, which means you may want to convert it to &str first
+//!   - performance gain compared to `columns` is minimal, use it only if you *really* need to as it is less convenient
+
+#![deny(missing_docs)]
+
 extern crate rustc_serialize;
 
 pub mod columns;
@@ -212,14 +264,18 @@ impl Row {
 /// - Ok(false) if no issue but it reached end of buffer
 /// - Err(Error::UnescapeQuote) if a quote if found within the column
 macro_rules! consume_quote {
-    ($bytes: expr, $delimiter: expr, $in_quote: expr) => {
+    ($bytes: expr, $delimiter: expr, $in_quote: expr,
+     $start: expr, $buf: expr, $available: expr, $quote_count: expr) => {
         $in_quote = false;
         loop {
             match $bytes.next() {
                 Some((_, &b'\"')) => {
                     match $bytes.clone().next() {
-                        Some((_, &b'\"')) => {
+                        Some((i, &b'\"')) => {
                             $bytes.next(); // escaping quote
+                            let _ = $buf.write(&$available[$start..i]);
+                            $start = i + 1;
+                            $quote_count += 1;
                         },
                         None | Some((_, &b'\r')) | Some((_, &b'\n')) => break,
                         Some((_, d)) if *d == $delimiter => break,
@@ -242,6 +298,7 @@ fn read_line<R: BufRead>(r: &mut R, buf: &mut Vec<u8>,
     let mut read = 0;
     let mut in_quote = false;
     let mut done = false;
+    let mut quote_count = 0;
     while !done {
         let used = {
             let available = match r.fill_buf() {
@@ -252,10 +309,11 @@ fn read_line<R: BufRead>(r: &mut R, buf: &mut Vec<u8>,
             };
             
             let mut bytes = available.iter().enumerate();
+            let mut start = 0;
 
             // previous buffer was exhausted without exiting from quotes
             if in_quote {
-                consume_quote!(bytes, delimiter, in_quote);
+                consume_quote!(bytes, delimiter, in_quote, start, buf, available, quote_count);
             }
 
             // use a simple loop instead of for loop to allow nested loop
@@ -264,23 +322,23 @@ fn read_line<R: BufRead>(r: &mut R, buf: &mut Vec<u8>,
                 match bytes.next() {
                     Some((i, &b'\"')) => {
                         if i == 0 || available[i - 1] == delimiter {
-                            consume_quote!(bytes, delimiter, in_quote);
+                            consume_quote!(bytes, delimiter, in_quote, start, buf, available, quote_count);
                         } else {
                             return Err(Error::UnexpextedQuote);
                         }
                     },
                     Some((i, &b'\n')) => {
-                        let _ = buf.write(&available[..i]);
                         done = true;
                         used = i + 1;
+                        let _ = buf.write(&available[start..i]);
                         break;
                     },
                     Some((i, &d)) => {
-                        if d == delimiter { cols.push(read + i); }
+                        if d == delimiter { cols.push(read + i - quote_count); }
                     },
                     None => {
-                        let _ = buf.write(available);
                         used = available.len();
+                        let _ = buf.write(&available[start..used]);
                         break;
                     },
                 }
